#include "seqfeti.h"
#include "gmatrix.h"
#include <limits.h>

seqfeti::seqfeti ()
{
  //  type of FETI implementation
  //  this means no type is selected
  fetiimpl=no_impl;
  //  type of storage of subdomain matrices
  smst=skyline_matrix;
  //smst=dense_matrix;

  //  number of subdomains
  ns=0;
  //  estimated number of rigid body modes (estimated dimension of the kernel)
  ense=0;
  

  //  maximum number of degrees of freedom on one subdomain
  ndofmax=0;
  //  number of DOFs (unknowns) in coarse problem
  ndofcp=0;
  //  threshold for kernel detection
  thresh=0.0;
  //  maximum number of iterations in conjugate gradient method
  nicg=0;
  //  number of performed iterations in conjugate gradient method
  anicg=0;
  //  required error
  errcg=0.0;
  //  attained error
  aerrcg=0.0;
  //  computer zero
  zero=1.0e-100;
  
  //  type of preconditioner
  prec=noprecond;
  //  size of matrix H
  hsize=0;
  //  maximum number of degrees of freedom on one subdomain
  ndofmax=0;
  //  number of DOFs (unknowns) in coarse problem
  ndofcp=0;
  
  
  //  numbers of DOFs on subdomains
  ndofmas=NULL;
  
  //  node-subdomain correspondence
  nsid=NULL;
  
  //  list of DOFs on subdomains
  //  it is used in connection with preconditioning
  cndom=NULL;
  
  //  array of numbers of unknowns (DOFs) contributing to the coarse problem
  ncdofd=NULL;
  
  //  array containing code numbers contributing to the coarse problem
  //  extracted values from subdomains to the coarse problem
  edofs=NULL;
  
  //  array containing numbers of RBM on subdomains
  nrbmdom=NULL;

  //  rigid body modes / kernel
  rbmdom=NULL;
  
  //  array containing addresses of first RBM in coarse matrix
  rbmadr=NULL;

  //  list of linearly dependent equations
  se=NULL;

  //  %matrix H
  h=NULL;
  
  //  number of contributions in the arrays booldatar, booldatac and booldata
  ncbool=NULL;

  //  array containing row indices for construction of the Boolean matrices
  booldatar=NULL;

  //  array containing column indices for construction of the Boolean matrices
  booldatac=NULL;

  //  array containing %matrix entries of the Boolean matrices
  booldata=NULL;

  //  array of coarse code numbers
  ccn=NULL;


  //  subdomain matrices stored in the %skyline storage
  smsky=NULL;
  //  subdomain matrices stored in the %dense format
  smdm=NULL;

}

seqfeti::~seqfeti ()
{
  long i;
  
  // ************************
  //  twodimensional arrays
  // ************************
  for (i=0;i<ns;i++){
    //  array of coarse code numbers
    delete [] ccn[i];
    //  array containing %matrix entries of the Boolean matrices
    delete [] booldata[i];
    //  array containing column indices for construction of the Boolean matrices
    delete [] booldatac[i];
    //  array containing row indices for construction of the Boolean matrices
    delete [] booldatar[i];
    //  list of linearly dependent equations
    delete [] se[i];
    //  rigid body modes / kernel
    delete [] rbmdom[i];
    //  array containing code numbers contributing to the coarse problem
    delete [] edofs[i];
    //  list of DOFs on subdomains
    delete [] cndom[i];
  }

  //  array of coarse code numbers
  delete [] ccn;
  //  array containing %matrix entries of the Boolean matrices
  delete [] booldata;
  //  array containing column indices for construction of the Boolean matrices
  delete [] booldatac;
  //  array containing row indices for construction of the Boolean matrices
  delete [] booldatar;
  //  list of linearly dependent equations
  delete [] se;
  //  rigid body modes / kernel
  delete [] rbmdom;
  //  array containing code numbers contributing to the coarse problem
  delete [] edofs;
  //  list of DOFs on subdomains
  delete [] cndom;
  
  
  // ************************
  //  onedimensional arrays
  // ************************
  //  number of contributions in the arrays booldatar, booldatac and booldata
  delete [] ncbool;
  //  %matrix H
  delete [] h;
  //  array containing addresses of first RBM in coarse matrix
  delete [] rbmadr;
  //  array containing numbers of RBM on subdomains
  delete [] nrbmdom;
  //  array of numbers of unknowns (DOFs) contributing to the coarse problem
  delete [] ncdofd;
  //  node-subdomain correspondence
  delete [] nsid;
  //  numbers of DOFs on subdomains
  delete [] ndofmas;
  
    
  for (i=0;i<ns;i++){
    smsky[i].~skyline ();
    smdm[i].~densemat ();
  }
}

/**
   function reads basic data
   
   @param top - pointer to general topology
   @param in - input file
   
   JK, 20.11.2007
*/
void seqfeti::read (gtopology *top,XFILE *in)
{
  //  type of implementation
  //  fetiimpl=1 - boolean matrices
  //  fetiimpl=2 - nonredundant definition of multipliers
  //  fetiimpl=3 - redundant definition of multipliers
  xfscanf (in,"%m",&fetiimplem_kwdset,&fetiimpl);
  
  if (fetiimpl!=boolean_matrices && fetiimpl!=nonredundant && fetiimpl!=redundant){
    print_err("unknown type of sequential FETI implementation",__FILE__,__LINE__,__func__);
  }
  
  //  the number of subdomains
  //  the number of estimated rigid body modes on one subdomain
  //  threshold for zero detection on the diagonal (threshold for detection of dependent row)
  //  the maximum number of iterations in the conjugate gradient method
  //  required norm of residual
  xfscanf (in,"%ld %ld %le %ld %le",&ns,&ense,&thresh,&nicg,&errcg);
  //  type of preconditioner
  xfscanf (in,"%d",(int*)&prec);
  
  
  // **********************************
  //  special requirements of solvers
  // **********************************
  if (fetiimpl==nonredundant || fetiimpl==redundant){
    //  sequential topology will be read in function mechtop or transtop :: read
    top->rst=1;
  }
  if (fetiimpl==boolean_matrices){
    //  sequential topology will be read in function mechtop or transtop :: read
    //  Boolean matrices will be read too
    top->rst=2;
  }
  if (prec==feti_dirichlet){
    //  code numbers will be generated with respect to the Schur complement method
    //  internal DOFs are ordered first, interface DOFs are ordered afterwards
    top->cngen=2;
  }
  
  top->cngen=0;
  
}

/**
   function prints basic data
   
   @param out - output file
   
   JK, 24.11.2008
*/
void seqfeti::print (FILE *out)
{
  //  type if implementation
  fprintf (out,"%d",fetiimpl);
  
  //  the number of subdomains
  //  the number of estimated rigid body modes on one subdomain
  //  threshold for zero detection on the diagonal (threshold for detection of dependent row)
  //  the maximum number of iterations in the conjugate gradient method
  //  required norm of residual
  fprintf (out,"%ld %ld %le %ld %le\n",ns,ense,thresh,nicg,errcg);
  //  type of preconditioner
  fprintf (out,"%d\n",prec);
}

/**
   function reads data for construction of Boolean matrices
   
   @param in - input file

   JK, 19.2.2008
*/
void seqfeti::read_booldata (XFILE *in)
{
  long i,j;
  
  //  allocation of array
  ncbool = new long [ns];
  booldatar = new long* [ns];
  booldatac = new long* [ns];
  booldata = new double* [ns];
  
  for (i=0;i<ns;i++){
    //  loop over the number of subdomains
    
    //  the number of contributions on the i-th subdomain
    xfscanf (in,"%ld",ncbool+i);
    
    //  allocation of array
    booldatar[i] = new long [ncbool[i]];
    booldatac[i] = new long [ncbool[i]];
    booldata[i] = new double [ncbool[i]];
    
    for (j=0;j<ncbool[i];j++){
      xfscanf (in,"%ld %ld %lf",&booldatar[i][j],&booldatac[i][j],&booldata[i][j]);
    }
  }

}

/**
   function assembles the following arrays
   ncdofd - array of numbers of unknowns (DOFs) contributing to the coarse problem
   wscalmat - 
   edofs - array containing code numbers contributing to the coarse problem
   ccn - array of coarse code numbers
   
   @param selnodfeti - pointer to the object containing selected nodes
   @param top - pointer to object with topology
   @param out - output file
   
   JK, 17.9.2007
*/
void seqfeti::initiate (seqselnodes *selnodfeti,gtopology *top,FILE *out)
{
  long i,j,k,ndofn,dof,nid;
  long *red;
  red = new long [ns+1];
  
  //  number of DOFs (unknowns) in coarse problem = total number of DOFs in selected nodes
  ndofcp = selnodfeti->tndofsn;
  
  //  array of numbers of unknowns (DOFs) contributing to the coarse problem
  ncdofd = new long [ns];
  for (i=0;i<ns;i++){
    ncdofd[i] = selnodfeti->snndofmas[i];
  }
  
  //  array containing local code numbers contributing to the coarse problem
  edofs = new long* [ns];
  for (i=0;i<ns;i++){
    edofs[i] = new long [ncdofd[i]];
  }
  
  for (i=0;i<ns;i++){
    for (j=0;j<ncdofd[i];j++){
      edofs[i][j] = selnodfeti->lndofmas[i][j];
    }
  }
  //  at this moment, edofs contains DOFs in global glued ordering
  //  nodes are taken into account in increasing order and code numbers
  //  are generated, the ordering has to be recalculated from the global
  //  glued ordering to local ordering, this is done with the help of
  //  the arry red
  
  red[0]=0;
  nid=0;
  //  loop over the number of subdomains
  for (i=0;i<ns;i++){
    red[i+1]=0;
    //  loop over the number of nodes in the i-th subdomain
    for (j=0;j<top->stop->nnsd[i];j++){
      ndofn = top->give_ndofn (nid);
      for (k=0;k<ndofn;k++){
	dof = top->give_dof (nid,k);
	if (dof>red[i+1]){
	  red[i+1]=dof;
	}
      }
      nid++;
    }
  }
  
  //  array red contains the minimum numbers used in subdomains
  //  these numbers have to be subtracted from all code numbers
  
  for (i=0;i<ns;i++){
    for (j=0;j<ncdofd[i];j++){
      if (edofs[i][j]>0)
	edofs[i][j] -= red[i];
      else
	edofs[i][j] += red[i];
    }
  }
  
  
  fprintf (out,"\n\n\n check of the array edofs \n");
  for (i=0;i<ns;i++){
    fprintf (out,"\n domain %ld",i);
    for (j=0;j<ncdofd[i];j++){
      fprintf (out,"\n edofs  %6ld  %6ld",j,edofs[i][j]);
    }
  }
  
  
  


  // scaling matrix
  wscalmat = new double [ndofcp]; 
  for (i = 0; i < ndofcp; i++){
    //wscalmat[i] = double(selnodfeti->dofmultip[i]);
  }
  




  ccn = new long* [ns];
  for (i=0;i<ns;i++){
    ccn[i] = new long [ncdofd[i]];
  }
  
  for (i=0;i<ns;i++){
    for (j=0;j<ncdofd[i];j++){
      ccn[i][j] = selnodfeti->cndofmas[i][j];
    }
  }
  
  delete [] red;

  /*
  // lokalni wscalmat
  lwscalmat = new double* [ns]; 
  nlwscalmat = new long [ns];
  for (i = 0; i < ns; i++){
    nlwscalmat[i] = selnodfeti->ndofmas[i];
    lwscalmat[i] = new double [ nlwscalmat[i]]; 
    for (j = 0; j <  nlwscalmat[i]; j++){
      lwscalmat[i][j] = double(selnodfeti->ldofmultip[i][j]);
    }
  }
  */
}


/**
   function determines ndofmax
   the variable is used for simple allocation in other functions
   
   JK, 16.9.2007, checked 20.9.2009
*/
void seqfeti::det_ndofmax ()
{
  long i,j;
  
  if (fetiimpl==nonredundant || fetiimpl==redundant){
    ndofmax=0;
    for (i=0;i<ns;i++){
      if (ndofmax<ndofmas[i])
	ndofmax=ndofmas[i];
    }
  }
  
  if (fetiimpl==boolean_matrices){
    ndofmax=0;
    ndofcp=0;
    for (i=0;i<ns;i++){
      for (j=0;j<ncbool[i];j++){
	if (ndofmax<booldatac[i][j])
	  ndofmax=booldatac[i][j];
	if (ndofcp<booldatar[i][j])
	  ndofcp=booldatar[i][j];
      }
    }
  }
}

/**
   function assembles list of unknowns belonging to the subdomains
   
   function assembles:
   sid - node-subdomain correspondence
   ndofmas - numbers of DOFs on subdomains
   cndom - list of DOFs on subdomains
   
   @param top - pointer to topology
   @param out - output file
   
   JK, 21.8.2007
*/
void seqfeti::assemble_subdom_unknowns (gtopology *top,FILE *out)
{
  long i,j,k,l,m,ndofn;

  //  node-subdomain correspondence
  nsid = new long [top->nn];
  l=0;
  for (i=0;i<ns;i++){
    for (j=0;j<top->stop->nnsd[i];j++){
      nsid[l]=i;
      l++;
    }
  }
  
  
  //  number of DOFs on subdomains
  ndofmas = new long [ns];
  for (i=0;i<ns;i++){
    ndofmas[i]=0;
  }
  
  for (i=0;i<top->nn;i++){
    //  number of DOFs in node
    ndofn=top->give_ndofn (i);
    //  subdomain id
    l=nsid[i];
    for (j=0;j<ndofn;j++){
      //  DOF id
      k=top->give_dof (i,j);
      if (k>0)
	ndofmas[l]++;
    }
  }
  
  //  list of DOFs on subdomains
  cndom = new long* [ns];
  for (i=0;i<ns;i++){
    cndom[i] = new long [ndofmas[i]];
    ndofmas[i]=0;
  }
  
  for (i=0;i<top->nn;i++){
    //  number of DOFs in node
    ndofn=top->give_ndofn (i);
    //  subdomain id
    l=nsid[i];
    for (j=0;j<ndofn;j++){
      //  DOF id
      k=top->give_dof (i,j);
      if (k>0){
	cndom[l][ndofmas[l]]=k;
	ndofmas[l]++;
      }
    }
  }
  
  

  
  
  if (prec == feti_dirichlet){
    for (i=0;i<ns;i++){
      for (j=0;j<ndofmas[i];j++){
	l=cndom[i][j];
	m=j;
	for (k=j;k<ndofmas[i];k++){
	  if (l>cndom[i][k]){
	    l=cndom[i][k];
	    m=k;
	  }
	}
	cndom[i][m]=cndom[i][j];
	cndom[i][j]=l;
      }
    }
  }

  
  fprintf (out,"\n\n kontrola pole ndofmas");
  for (i=0;i<ns;i++){
    fprintf (out,"\n ndofmas %6ld   %6ld",i,ndofmas[i]);
  }
  fprintf (out,"\n\n kontrola pole cndom");
  for (i=0;i<ns;i++){
    fprintf (out,"\n domain %ld",i);
    for (j=0;j<ndofmas[i];j++){
      fprintf (out,"\n dom %6ld  dof %6ld   %6ld",i,j,cndom[i][j]);
    }
  }  
  


  jum = new double* [ns];
  for (i=0;i<ns;i++){
    jum[i] = new double [ndofmas[i]];
  }
  
  for (i=0;i<ns;i++){
    for (j=0;j<ndofmas[i];j++){
      jum[i][j]=0.0;
    }
  }

}


/**
   function assembles subdomain matrices from the %matrix of the whole system
   
   @param gm - pointer to the %matrix of the system
   @param out - output file
   
   JK, 21.8.2007
*/
void seqfeti::subdomain_matrices (gmatrix *gm,FILE *out)
{
  long i,j,k;
  
  if (smst==dense_matrix){
    smdm = new densemat [ns];
    for (i=0;i<ns;i++){
      //smdm[i].assemble_dense_from_scr (gm->scr->adr,gm->scr->ci,gm->scr->a,ndofmas[i],cndom[i]);
    }
  }
  if (smst==skyline_matrix){
    smsky = new skyline [ns];
    for (i=0;i<ns;i++){
      smsky[i].assemble_from_scr (gm->scr->adr,gm->scr->ci,gm->scr->a,ndofmas[i],cndom[i]);
    }
  }
  
    
  if (fetiimpl==nonredundant || fetiimpl==redundant){
    long *aux;
    ndofprec = new long [ns];
    cnprec = new long* [ns];
    cpreccn = new long* [ns];
    
    for (i=0;i<ns;i++){
      aux = new long [ndofmas[i]];
      for (j=0;j<ndofmas[i];j++){
	aux[j]=0;
      }
      
      for (j=0;j<ncdofd[i];j++){
	k=edofs[i][j];
	if (k<0)
	  k=0-k-1;
	else
	  k--;
	if (k>-1)
	  aux[k]++;
      }
      
      ndofprec[i]=0;
      for (j=0;j<ndofmas[i];j++){
	if (aux[j]>0)
	  ndofprec[i]++;
      }
      
      fprintf (out,"\n\n aux  domena %ld\n",i);
      for (j=0;j<ndofmas[i];j++){
	fprintf (out," %ld",aux[j]);
      }
      
      
      cnprec[i] = new long [ndofprec[i]];
      cpreccn[i] = new long [ndofprec[i]];
      k=0;
      for (j=0;j<ndofmas[i];j++){
	if (aux[j]>0){
	  cnprec[i][k]=cndom[i][j];
	  cpreccn[i][k]=j;
	  k++;
	}
      }
      
      delete [] aux;
    }
  
    fprintf (out,"\n\n\n\n kontrola ve funkci subdomain_matrices (file %s, line %d)\n\n",__FILE__,__LINE__);
    for (i=0;i<ns;i++){
      fprintf (out,"\n domena %5ld     ndofprec %ld\n",i,ndofprec[i]);
      for (j=0;j<ndofprec[i];j++){
	fprintf (out,"  %ld",cnprec[i][j]);
      }
      fprintf (out,"\n");
      for (j=0;j<ndofprec[i];j++){
	fprintf (out,"  %ld",cpreccn[i][j]);
      }
    }
    fprintf (out,"\n\n\n\n\n");
  }
  

  if (fetiimpl==nonredundant || fetiimpl==redundant){
    if (prec==feti_lumped){
      smscr = new symcomprow [ns];
      for (i=0;i<ns;i++){
	//gm->scr->select_submatrix (&smscr[i],ndofmas[i],cndom[i]);
	gm->scr->select_submatrix (&smscr[i],ndofprec[i],cnprec[i]);
      }
    }
    
    if (prec==feti_dirichlet){
      psmdm = new densemat [ns];
      
      double *x,*y,*condvect;
      for (i=0;i<ns;i++){
	x = new double [ndofmas[i]];
	y = new double [ndofmas[i]];
	psmdm[i].a = new double [ndofprec[i]*ndofprec[i]];
	psmdm[i].negm=ndofprec[i]*ndofprec[i];
	psmdm[i].n=ndofprec[i];
	condvect=new double [ndofprec[i]];
	
	smsky[i].ldlkon_sky (psmdm[i].a,condvect,x,y,ndofprec[i],1);
	smsky[i].~skyline ();
	
	delete [] x;
	delete [] y;
	delete [] condvect;
      }
      
      smsky = new skyline [ns];
      for (i=0;i<ns;i++){
	smsky[i].assemble_from_scr (gm->scr->adr,gm->scr->ci,gm->scr->a,ndofmas[i],cndom[i]);
      }
      
    }
  }
  
  if (fetiimpl==boolean_matrices){
    if (prec==feti_lumped){
      smskyprec = new skyline [ns];
      for (i=0;i<ns;i++){
	smskyprec[i].assemble_from_scr (gm->scr->adr,gm->scr->ci,gm->scr->a,ndofmas[i],cndom[i]);
      }
    }
    if (prec==feti_dirichlet){
      
      ndofprec = new long[ns];
      for (i=0;i<ns;i++){
	ndofprec[i] = 1;
	for(j = 1; j < ncbool[i]; j++){
	  if(booldatac[i][j] != booldatac[i][j-1]){
	    ndofprec[i]++;
	  }
	}
      }
      
      psmdm = new densemat [ns];
      
      double *x,*y,*condvect;
      for (i=0;i<ns;i++){
	x = new double [ndofmas[i]];
	y = new double [ndofmas[i]];
	psmdm[i].a = new double [ndofprec[i]*ndofprec[i]];
	psmdm[i].negm=ndofprec[i]*ndofprec[i];
	psmdm[i].n=ndofprec[i];
	condvect=new double [ndofprec[i]];
	
	smsky[i].ldlkon_sky (psmdm[i].a,condvect,x,y,ndofprec[i],1);
	smsky[i].~skyline ();
	
	delete [] x;
	delete [] y;
	delete [] condvect;
      }
      
      smsky = new skyline [ns];
      for (i=0;i<ns;i++){
	smsky[i].assemble_from_scr (gm->scr->adr,gm->scr->ci,gm->scr->a,ndofmas[i],cndom[i]);
      }
    }
  }  
  
  //fclose (out);
  //abort ();

  /*
  delete [] ndofprec;
  for (i=0;i<ns;i++){
    delete [] cnprec[i];
  }
  delete [] cnprec;
  */
}

/**
   function computes kernel (rigid body modes) of the %matrix
   
   JK, 28.8.2007
*/
void seqfeti::kernel (FILE *out)
{
  long i,j;
  
  //  array for numbers of rigid body modes
  if (nrbmdom!=NULL){
    delete [] nrbmdom;
  }
  nrbmdom = new long [ns];
  
  //  array for linearly dependent equation numbers
  if (se!=NULL){
    for (i=0;i<ns;i++){
      delete [] se[i];
    }
    delete [] se;
  }
  se = new long* [ns];
  for (i=0;i<ns;i++){
    se[i] = new long [ense];
    for (j=0;j<ense;j++){
      se[i][j]=-1;
    }
  }
  
  //  array for rigid body modes
  if (rbmdom!=NULL){
    for (i=0;i<ns;i++){
      delete [] rbmdom[i];
    }
    delete [] rbmdom;
  }
  rbmdom = new double* [ns];
  for (i=0;i<ns;i++){
    rbmdom[i] = new double [ense*ndofmas[i]];
    for (j=0;j<ense*ndofmas[i];j++){
      rbmdom[i][j]=0.0;
    }
  }
  
  
  if (smst==dense_matrix){
    //  loop over subdomains
    for (i=0;i<ns;i++){
      smdm[i].ker (rbmdom[i],nrbmdom[i],se[i],ense,thresh);
    }
  }
  if (smst==skyline_matrix){
    //  loop over subdomains
    for (i=0;i<ns;i++){
      smsky[i].ker (rbmdom[i],nrbmdom[i],se[i],ense,thresh,3);
      
      //smsky[i].printmat (out);
    }
  }

}
 













/**
   function assembles the Boolean %matrix of one subdomain
   
   @param sdid - subdomain id
   @param a - array storing the Boolean %matrix

   JK, 17.2.2009
*/
void seqfeti::boolean_matrix (long sdid,double *a)
{
  long i,ri,ci;
  
  for (i=0;i<ncbool[sdid];i++){
    ri=booldatar[sdid][i]-1;
    ci=booldatac[sdid][i]-1;
    a[ri*ndofmas[sdid]+ci]=booldata[sdid][i];
  }
}



/**
   function extracts contributions from local %vector to coarse %vector
   
   @param nd - domain id (number of required subdomain)
   @param lv - local %vector
   @param cv - coarse %vector
   
   JK, 14.9.2007
*/
void seqfeti::local_coarse (long nd,double *lv,double *cv)
{
  if (fetiimpl==nonredundant || fetiimpl==redundant){
    
    long i,j,k;
    
    for (i=0;i<ncdofd[nd];i++){
      j=edofs[nd][i];
      k=ccn[nd][i]-1;
      if (j<0){
	j=0-j-1;
	cv[k]-=lv[j];
      }
      else{
	j=j-1;
	cv[k]+=lv[j];
      }
    }
  }
  
  if (fetiimpl==boolean_matrices){
    
    double *a,*av;
    
    //  allocation of Boolean matrix
    a = new double [ndofcp*ndofmas[nd]];
    nullv (a,ndofcp*ndofmas[nd]);
    
    //  allocation of auxiliary vector
    av = new double [ndofcp];
    nullv (av,ndofcp);
    
    //  assembling of Boolean matrix
    boolean_matrix (nd,a);
    //  Bu
    mxv (a,lv,av,ndofcp,ndofmas[nd]);
    //
    addv (cv,av,ndofcp);
    
    delete [] a;
    delete [] av;
  }
  
}


/**
   function extracts contributions from coarse %vector to local %vector
   
   @param nd - domain id (number of required subdomain)
   @param lv - local %vector
   @param cv - coarse %vector

   JK, 14.9.2007
*/
void seqfeti::coarse_local (long nd,double *lv,double *cv)
{
  if (fetiimpl==nonredundant || fetiimpl==redundant){
    long i,j,k;
    
    for (i=0;i<ncdofd[nd];i++){
      k=ccn[nd][i]-1;
      j=edofs[nd][i];
      if (j<0){
	j=0-j-1;
	lv[j]-=cv[k];
      }
      else{
	j=j-1;
	lv[j]+=cv[k];
      }
    }
  }
  
  if (fetiimpl==boolean_matrices){
    
    double *a,*av;
    
    //  allocation of Boolean matrix
    a = new double [ndofcp*ndofmas[nd]];
    nullv (a,ndofcp*ndofmas[nd]);
    
    //  allocation of auxiliary vector
    av = new double [ndofmas[nd]];
    nullv (av,ndofmas[nd]);
    
    //  assembling of Boolean matrix
    boolean_matrix (nd,a);
    //  Bu
    mtxv (a,cv,av,ndofcp,ndofmas[nd]);
    //
    addv (lv,av,ndofmas[nd]);
    
    delete [] a;
    delete [] av;
  }
  
}







/**
   function computes size of %matrix H
   
   @param out - output stream
   
   JK, 16.9.2007
*/
void seqfeti::hmatrixsize (FILE *out)
{
  long i;
  
  //  addresses computed from nrbmdom array
  rbmadr = new long [ns+1];
  for (i=0;i<ns+1;i++){
    rbmadr[i]=0;
  }
  
  //  computation of addresses
  rbmadr[0]=0;
  for (i=0;i<ns;i++){
    rbmadr[i+1]=rbmadr[i]+nrbmdom[i];
  }
  
  //  size of the matrix H
  hsize=rbmadr[ns];
  
}



/**
   function assembles %matrix H
   columns are RBM after localization
   
   @param h - %matrix H
   @param rbm - array containing rigid body motions
   
   JK, 16.9.2007
*/
void seqfeti::hmatrix (double *h)
{
  long i,k,l;
  double *av;
  
  av = new double [ndofcp];
  
  for (i=0;i<ns;i++){
    for (k=0;k<nrbmdom[i];k++){
      nullv (av,ndofcp);
      local_coarse (i,rbmdom[i]+k*ndofmas[i],av);
      for (l=0;l<ndofcp;l++){
	h[l*hsize+rbmadr[i]+k]=0.0-av[l];
      }
    }
  }
  
  delete [] av;
}


/**
   function assembles %vector q
   
   @param q - %vector q
   @param rbm - array containing rigid body motions
   @param f - right hand side
   
   JK, 16.9.2007
*/
void seqfeti::qvector (double *q,double **ff)
{
  long i,j,k;
  
  k=0;
  for (i=0;i<ns;i++){
    for (j=0;j<nrbmdom[i];j++){
      q[k]=0.0-ss(&rbmdom[i][j*ndofmas[i]],ff[i],ndofmas[i]);
      k++;
    }
  }
  
}



/**
   function computes projection in FETI method
   v_new = v_old - H . (H^T . H)^{-1} . H^T . v_old
   function overwrites vector v by projected vector
   
   @param v - %vector
   @param h - %matrix H
   @param h1 - %matrix (H^T.H)^{-1}
   @param ndofcp,hsize - number of rows and columns of %matrix H
   
   ndofcp - number of Lagrange multipliers
   hsize - total number of all rigid body motions
   
   JK, 16.9.2007
*/
void seqfeti::feti_projection (double *v,double *h,double *h1)
{
  long i;
  double *p,*q,*w;
  
  p = new double [hsize];
  q = new double [hsize];
  w = new double [ndofcp];

  //  p = H^T.v
  //mtvc (h,v,p,ndofcp,hsize);
  mtxv (h,v,p,ndofcp,hsize);
  
  
  /*
  fprintf (out,"\n\n\n kontrola H^T.v\n");
  for (i=0;i<hsize;i++){
    fprintf (out," %lf",p[i]);
  }
  */

  
  //  q = (H^T.H)^{-1}.p
  mxv (h1,p,q,hsize,hsize);
  

  /*
  fprintf (out,"\n\n\n kontrola (H^T.H)^{-1}.H^T.v\n");
  for (i=0;i<hsize;i++){
    fprintf (out," %lf",q[i]);
  }
  */


  //  w = H.q
  mxv (h,q,w,ndofcp,hsize);
  
  
  /*
  fprintf (out,"\n\n\n kontrola H.(H^T.H)^{-1}.H^T.v\n");
  for (i=0;i<ndofcp;i++){
    fprintf (out," %lf",w[i]);
  }
  */
  
  //  v_new = v_old - w
  for (i=0;i<ndofcp;i++){
    v[i]-=w[i];
  }
  
  delete [] w;  delete [] q;  delete [] p;
}


/**
   function obtaines jumps from problem solved
   
   @param jump - array of jumps
   
   JK, 28.11.2007
*/
/*
void seqfeti::get_jumps (double *jump)
{
  long i,j;
  

  //if (jum==NULL){
  //jum = new double* [ns];
  //for (i=0;i<ns;i++){
  //  jum[i] = new double [ndofmas[i]];
  //}
  //}

  for (i=0;i<ns;i++){
    for (j=0;j<ndofmas[i];j++){
      jum[i][j]=0.0;
    }
  }
  
  
  //  allocation of vectors of right hand sides on subdomains
  for (i=0;i<ns;i++){
    globloc (jump,jum[i],cndom[i],ndofmas[i]);
  }
  
}
*/


void seqfeti::scaling(double *invect,double *outvect,long n,FILE *out)
{
  double *aux;
  long i;
  aux = new double[n];
  
  for(i = 0; i < n; i++){
    aux[i] = invect[i]/wscalmat[i];
  }
  
  /*
  fprintf (out,"\n\n\n kontrola skalovani \n");
  for(i = 0; i < n; i++){
    fprintf (out,"%ld  pred %le po %le wscalmat %lf\n",i+1,invect[i],aux[i],wscalmat[i]); 
  }
  */
  
  for(i = 0; i < n; i++){
    outvect[i] = aux[i];
  }
  
  delete []aux;
}

void seqfeti::lscaling(double *invect,double *outvect,long ndom,FILE *out)
{
  double *aux;
  long i;
  
  aux = new double[nlwscalmat[ndom]];
  
  for(i = 0; i < nlwscalmat[ndom]; i++){
    aux[i] = invect[i]/lwscalmat[ndom][i];
  }
  
  
  fprintf (out,"\n\n\n kontrola skalovani \n");
  for(i = 0; i <  nlwscalmat[ndom]; i++){
    fprintf (out,"%ld  pred %le po %le wscalmat %lf ccn %ld\n",i+1,invect[i],aux[i],lwscalmat[ndom][i],ccn[ndom][i]); 
  }
  
  
  for(i = 0; i <  nlwscalmat[ndom]; i++){
    outvect[i] = aux[i];
  }
  
  delete []aux;
}



/**

*/
void seqfeti::lumpedprec (long nd,double *dd,double *pp)
{
  long i;
  double *d,*p;
  
  if (fetiimpl == nonredundant || fetiimpl==redundant){
    d = new double [ndofprec[nd]];
    p = new double [ndofprec[nd]];
    
    for (i=0;i<ndofprec[nd];i++){
      d[i]=0.0;
      p[i]=0.0;
    }
    
    for (i=0;i<ndofprec[nd];i++){
      d[i]=dd[cpreccn[nd][i]];
    }
    
    smscr[nd].mxv_scr (d,p);
    
    for (i=0;i<ndofprec[nd];i++){
      pp[cpreccn[nd][i]]=p[i];
    }
    delete [] p;
    delete [] d;
  }
  if(fetiimpl == boolean_matrices){
    
    /*for (i=0;i<ndofmas[nd];i++){
      pp[i]=dd[i];
    }
    */
    smskyprec[nd].mxv_sky (dd,pp);
    
    
  }
}

/**
   JK, 7.5.2008
*/
void seqfeti::dirichletprec (long nd,double *dd,double *pp)
{
  long i,j;
  double *d,*p;
  
  if (fetiimpl == nonredundant || fetiimpl==redundant){
    d = new double [ndofprec[nd]];
    p = new double [ndofprec[nd]];
    
    for (i=0;i<ndofprec[nd];i++){
      d[i]=0.0;
      p[i]=0.0;
    }
    
    for (i=0;i<ndofprec[nd];i++){
    d[i]=dd[cpreccn[nd][i]];
    }
    
    psmdm[nd].mxv_dm (d,p);
    
    
    for (i=0;i<ndofprec[nd];i++){
      pp[cpreccn[nd][i]]=p[i];
    }
    
    delete [] p;
    delete [] d;
  }
  if (fetiimpl == boolean_matrices){
    d = new double [ndofprec[nd]];
    p = new double [ndofprec[nd]];
    j = 0;
    for (i=0;i<ndofmas[nd];i++){
      if(dd[i] != 0){
	d[j]=dd[i];
	j++;
      }
    }
    psmdm[nd].mxv_dm (d,p);
    
    j = 0;
    for (i=0;i<ndofmas[nd];i++){
      if(dd[i] != 0){
	pp[i]=p[j];
	j++;
      }
      else{
	pp[i] = 0;
      }
    }
  }
}



/**
   function performs modified conjugate gradient method
   
   @param w - %vector of Lagrange multipliers
   @param ff - %vectors of right hand side
   @param h - %matrix H
   @param h1 - %matrix (H^T.H)^{-1}
   @param ndofcp,hsize - number of rows and columns of %matrix H
   
   ndofcp - number of Lagrange multipliers
   hsize - total number of all rigid body motions
   
   JK, 16.9.2007
*/
void seqfeti::mpcg (gtopology *top,gmatrix *gm,
		    double *w,double **ff,double *q,double *h,double *h1,FILE *out)
{
  long i,j,k;
  double nom,denom,alpha,beta;
  double *d,*dd,*g,*p,*pp,*hq;

  // JB
  //prepare_scaling(top,out);
  
  //  vectors defined on each subdomain
  dd = new double [ndofmax];
  pp = new double [ndofmax];

  
  //  direction vector allocation
  d = new double [ndofcp];
  //  residuum vector allocation
  g = new double [ndofcp];
  //  auxiliary vector allocation
  p = new double [ndofcp];
  
  //fprintf (stdout,"\n\n\n ndofcp %ld       hsize  %ld",ndofcp,hsize);
  
  //  initiation of conjugate gradients
  
  hq = new double [hsize];
  //  (H^T.H)^{-1} q
  mxv (h1,q,hq,hsize,hsize);
  //  H [(H^T.H)^{-1} q]
  mxv (h,hq,w,ndofcp,hsize);
  delete [] hq;
  
  nullv (g,ndofcp);
  for (j=0;j<ns;j++){
    nullv (dd,ndofmas[j]);
    coarse_local (j,dd,w);
    // f - \lambda_0
    subv (dd,ff[j],ndofmas[j]);
    
    fprintf (out,"\n");
    for (i=0;i<ndofmas[j];i++){
      fprintf (out,"\n zoufalec %4ld    %20.15e",i,dd[i]);
    }

    //  K^+ (f - \lambda_0) = r
    if (smst==dense_matrix)
      break;
    if (smst==skyline_matrix)
      smsky[j].ldl_feti_sky (pp,dd,nrbmdom[j],se[j],zero);
    
    fprintf (out,"\n");
    for (i=0;i<nrbmdom[j];i++){
      fprintf (out,"\n zoufalec    se %4ld",se[j][i]);
    }
    fprintf (out,"\n");
    for (i=0;i<ndofmas[j];i++){
      fprintf (out,"\n zoufalec %4ld    %20.15e",i,pp[i]);
    }
    
    //  zacatek - streda 28.11.2007
    //  odecet skoku
    subv (pp,jum[j],ndofmas[j]);
    //  konec - streda 28.11.2007
    
    
    //nullvr (buff,maxlggl+1);
    local_coarse (j,pp,g);
  }
  
  fprintf (out,"\n");
  for (i=0;i<ndofcp;i++){
    fprintf (out,"\n jouda zoufalec %4ld    %20.15e",i,g[i]);
  }
  
  //  P r
  feti_projection (g,h,h1);
  
  //  initialization of direction vector
  for (i=0;i<ndofcp;i++){
    d[i]=0.0-g[i];
  }
  
  //  nominator evaluation
  nom = ss (g,g,ndofcp);
  
  
  
  // *************************
  //  main iteration loop   **
  // *************************
  for (i=0;i<nicg;i++){
    
    // ******************************************
    //   auxiliary vector computation K.d = p  **
    // ******************************************
    nullv (p,ndofcp);
    for (j=0;j<ns;j++){
      nullv (dd,ndofmax);
      coarse_local (j,dd,d);
      
      //  computation of K^+ d on master
      if (smst==dense_matrix)
	break;
      if (smst==skyline_matrix)
	smsky[j].ldl_feti_sky (pp,dd,nrbmdom[j],se[j],zero);
      
      //nullvr (buff,maxlggl+1);
      local_coarse (j,pp,p);
    }
    
    //  denominator of alpha
    denom = ss (d,p,ndofcp);
    
    //fprintf (stdout,"\n denominator u alpha   %le",denom);
    
    fprintf (out,"\n\n kontrola citatele a jmenovatele pred alpha %e / %e",nom,denom);
    //fprintf (stderr,"\n\n kontrola citatele a jmenovatele pred alpha %lf / %lf",nom,denom);
    
    if (fabs(denom)<zero){
      fprintf (stderr,"\n\n zero denominator in modified conjugate gradient method (file %s, line %d).\n",__FILE__,__LINE__);
      break;
    }
    
    // *******************
    //   vypocet alpha  **
    // *******************
    alpha = nom/denom;
    
    // **************************************************************
    //  vypocet noveho gradientu g a nove aproximace neznamych x   **
    // **************************************************************
    for (j=0;j<ndofcp;j++){
      w[j]+=alpha*d[j];
      g[j]+=alpha*p[j];
    }
    
    feti_projection (g,h,h1);
    
    
    // *******************************************************
    // *******************************************************
    //  vlozka s predpodminenim
    // *******************************************************
    // *******************************************************
    
    nullv (p,ndofcp);
    
    switch (prec){
    case noprecond:{
      for (j=0;j<ns;j++){
	nullv (dd,ndofmax);
	coarse_local (j,dd,g);
	nullv (pp,ndofmax);
	for (k=0;k<ndofmas[j];k++){
	  pp[k]=dd[k];
	}
	local_coarse (j,pp,p);
      }
      break;
    }
    case feti_lumped:{
      //scaling(g,g,ndofcp,out);
      for (j=0;j<ns;j++){
	nullv (dd,ndofmax);
	coarse_local (j,dd,g);
	//lscaling(dd,dd,j,out);
	nullv (pp,ndofmax);
	
	fprintf (out,"\n\nkontrola v lumpedprec\n");
	for (k=0;k<ndofmas[j];k++){
	  fprintf (out,"%lf\n",dd[k]);
	}
	
	lumpedprec (j,dd,pp);
		
	//lscaling(pp,pp,j,out);
	local_coarse (j,pp,p);
      }
      //scaling(p,p,ndofcp,out);
      break;
    }
    case feti_dirichlet:{
      //scaling(g,g,ndofcp,out);
      for (j=0;j<ns;j++){
	nullv (dd,ndofmax);
	coarse_local (j,dd,g);
	//lscaling(dd,dd,j,out);
	nullv (pp,ndofmax);
	
	fprintf (out,"\n\nkontrola v dirichlet\n");
	for (k=0;k<ndofmas[j];k++){
	  fprintf (out,"%lf\n",dd[k]);
	}
	dirichletprec (j,dd,pp);
	//for (k=0;k<ndofmas[j];k++){
	//pp[k]=dd[k];
	//}
	//lscaling(pp,pp,j,out);
	local_coarse (j,pp,p);
      }
      //scaling(p,p,ndofcp,out);
      break;
    }
    default:{
      fprintf (stderr,"\n\n unknown type of preconditioner is required in function mpcg (file %s, line %d)\n",__FILE__,__LINE__);
    }
    }
    
    feti_projection (p,h,h1);
    
    // *******************************************************
    // *******************************************************
    //   konec vlozky s predpodminenim
    // *******************************************************
    // *******************************************************
    
    
    denom = nom;
    if (fabs(denom)<zero){
      fprintf (stderr,"\n\n zero denominator of beta in function (file %s, line %d).\n",__FILE__,__LINE__);
      break;
    }
    
    //nom = ss (g,g,ndofcp);
    nom=ss(p,g,ndofcp);
    
    fprintf (stdout,"\n iteration  %4ld        norm g   %e",i,sqrt(nom));
    fprintf (out,"\n iteration  %4ld        norm g   %e",i,sqrt(nom));
    
    fprintf (out,"\n kontrola citatele a jmenovatele pred betou  %e / %e",nom,denom);
    //fprintf (stderr,"\n kontrola citatele a jmenovatele pred betou  %lf / %lf",nom,denom);
    
    if (sqrt(nom)<errcg){
      break;
    }
    
    
    //  computation of beta coefficient
    beta = nom/denom;
    
    //  new direction vector
    for (j=0;j<ndofcp;j++){
      d[j]=beta*d[j]-p[j];
    }
    
  }
  
  anicg=i;  aerrcg=nom;
  
  fprintf (out,"\n\n\n\n kontrola Lagrangeovych multiplikatoru \n");
  for (i=0;i<ndofcp;i++){
    fprintf (out,"\n lagr. mult %4ld    %e",i,w[i]);
  }
  
  delete [] d;
  delete [] p;
  delete [] g;
  
  delete [] dd;
  delete [] pp;
}


/**
   function performs modified preconditioned reorthonormalized conjugate gradient method
   
   @param w - %vector of Lagrange multipliers
   @param ff - %vectors of right hand side
   @param h - %matrix H
   @param h1 - %matrix (H^T.H)^{-1}
   @param ndofcp,hsize - number of rows and columns of %matrix H
   
   ndofcp - number of Lagrange multipliers
   hsize - total number of all rigid body motions
   
   JK, 5.5.2008
*/
void seqfeti::mprcg (gtopology *top,gmatrix *gm,
		     double *lambda,double **ff,double *e,double *g,double *g1,FILE *out)
{
  long i,j,k;
  double nom,denom,alpha,s;
  double *ge,*r,*lambdad,*rr,*w,*p,*y,*op,*fp,*pp,*fpd,*ofp,*coef,*wd,*yd,*beta;

    r = new double [ndofcp];
  lambdad = new double [ndofmax];
  rr = new double [ndofmax];
  w = new double [ndofcp];
  p = new double [ndofcp];
  y = new double [ndofcp];
  op = new double [ndofcp*101];
  fp = new double [ndofcp];
  pp = new double [ndofmax];
  fpd = new double [ndofmax];
  ofp = new double [ndofcp*101];
  coef = new double [101];
  wd = new double [ndofmax];
  yd = new double [ndofmax];
  beta = new double [101];


  //  vectors defined on each subdomain

  


  //  initiation of conjugate gradients
  
  ge = new double [hsize];
  //  (G^T.G)^{-1} e
  mxv (g1,e,ge,hsize,hsize);
  //  G [(G^T.G)^{-1} e]
  mxv (g,ge,lambda,ndofcp,hsize);
  delete [] ge;
  
  nullv (w,ndofcp);
  for (j=0;j<ns;j++){
    nullv (lambdad,ndofmas[j]);
    coarse_local (j,lambdad,lambda);
    // f - \lambda_0
    subv (lambdad,ff[j],ndofmas[j]);
    
    //  K^+ (f - \lambda_0) = r
    if (smst==dense_matrix)
      break;
    if (smst==skyline_matrix)
      smsky[j].ldl_feti_sky (wd,lambdad,nrbmdom[j],se[j],zero);
    
    
    //  zacatek - streda 28.11.2007
    //  odecet skoku
    //subv (pp,jum[j],ndofmas[j]);
    //  konec - streda 28.11.2007
    
    
    //nullvr (buff,maxlggl+1);
    local_coarse (j,wd,w);
  }
  
  for (j=0;j<ndofcp;j++){
    w[j]*=-1.0;
  }

  //  P r
  feti_projection (w,g,g1);
  
  //  initialization of direction vector
  for (i=0;i<ndofcp;i++){
    p[i]=w[i];
    y[i]=w[i];
  }

  for (j=0;j<ndofcp;j++){
    op[0*ndofcp+j]=p[j];
  }
  
  
  // *************************
  //  main iteration loop   **
  // *************************
  for (i=0;i<nicg;i++){
    
    // ******************************************
    //   auxiliary vector computation K.d = p  **
    // ******************************************
    nullv (fp,ndofcp);
    for (j=0;j<ns;j++){
      nullv (pp,ndofmax);
      coarse_local (j,pp,p);
      
      //  computation of K^+ d on master
      if (smst==dense_matrix)
	break;
      if (smst==skyline_matrix)
	smsky[j].ldl_feti_sky (fpd,pp,nrbmdom[j],se[j],zero);
      
      //nullvr (buff,maxlggl+1);
      local_coarse (j,fpd,fp);
    }
    
    for (j=0;j<ndofcp;j++){
      ofp[i*ndofcp+j]=fp[j];
    }
    
    nom = ss (p,w,ndofcp);

    //  denominator of alpha
    denom = ss (p,fp,ndofcp);
    
    coef[i]=denom;

    //fprintf (stdout,"\n denominator u alpha   %le",denom);
    
    fprintf (out,"\n\n kontrola citatele a jmenovatele pred alpha %e / %e",nom,denom);
    //fprintf (stderr,"\n\n kontrola citatele a jmenovatele pred alpha %lf / %lf",nom,denom);
    
    if (fabs(denom)<zero){
      fprintf (stderr,"\n\n zero denominator in modified conjugate gradient method (file %s, line %d).\n",__FILE__,__LINE__);
      break;
    }
    
    // *******************
    //   vypocet alpha  **
    // *******************
    alpha = nom/denom;
    
    // **************************************************************
    //  vypocet noveho gradientu g a nove aproximace neznamych x   **
    // **************************************************************
    nom=0.0;
    for (j=0;j<ndofcp;j++){
      lambda[j]+=alpha*p[j];
      w[j]-=alpha*fp[j];
      nom+=w[j]*w[j];
    }
    
    fprintf (stdout,"\n iteration  %4ld        norm g   %e",i,sqrt(nom));
    fprintf (out,"\n iteration  %4ld        norm g   %e",i,sqrt(nom));

    feti_projection (w,g,g1);
    
    // *******************************************************
    // *******************************************************
    //  vlozka s predpodminenim
    // *******************************************************
    // *******************************************************
    
    nullv (y,ndofcp);
    
    switch (prec){
    case noprecond:{
      for (j=0;j<ndofcp;j++){
	y[j]=w[j];
      }
      break;
    }
    case feti_lumped:{
      //scaling(w,w,ndofcp,out);
      for (j=0;j<ns;j++){
	nullv (wd,ndofmax);
	coarse_local (j,wd,w);
	//lscaling(dd,dd,j,out);
	nullv (pp,ndofmax);
	
	lumpedprec (j,wd,yd);
	
	//lscaling(pp,pp,j,out);
	local_coarse (j,yd,y);
      }
      //scaling(y,y,ndofcp,out);
      break;
    }
    default:{
      fprintf (stderr,"\n\n unknown type of preconditioner is required in function mpcg (file %s, line %d)\n",__FILE__,__LINE__);
    }
    }
    
    feti_projection (y,g,g1);
    
    // *******************************************************
    // *******************************************************
    //   konec vlozky s predpodminenim
    // *******************************************************
    // *******************************************************
    
    for (k=0;k<=i;k++){
      s=0.0;
      for (j=0;j<ndofcp;j++){
	s+=y[j]*ofp[k*ndofcp+j];
      }
      beta[k]=s/coef[k];
    }
    for (j=0;j<ndofcp;j++){
      s=0.0;
      for (k=0;k<=i;k++){
	s+=beta[k]*op[k*ndofcp+j];
      }
      p[j]=y[j]-s;
      op[(i+1)*ndofcp+j]=p[j];
    }
    
    
    if (sqrt(nom)<errcg){
      break;
    }
    
    
  }
  
  anicg=i;  aerrcg=nom;
  
  fprintf (out,"\n\n\n\n kontrola Lagrangeovych multiplikatoru \n");
  for (i=0;i<ndofcp;i++){
    //fprintf (out,"\n lagr. mult %4ld    %e",i,w[i]);
  }
  
}





/**
   function computes displacements from Lagrange multipliers
   function is used in FETI method
   
   @param w - array containing Lagrange multipliers
   @param d - array containing displacements
   @param f - array containing load forces
   @param rbm - array containing rigid body motions
   @param h - %matrix H (columns are rigid body motions)
   @param h1 - %matrix (H^T . H)^{-1}
   @param ndofcp - number of Lagrange multipliers
   @param hsize - number of columns of %matrix H
   
   JK, 16.9.2007
*/
void seqfeti::lagrmultdispl (gtopology *top,gmatrix *gm,
			     double *w,double **d,double **ff,
			     double *h,double *h1,FILE *out)
{
  long i,j,k,l;
  double *g,*dd,*pp,*gamma,*u,*av;
  
  
  //av = new double [maxnrbm];
  av = new double [6];

  pp = new double [ndofmax];
  dd = new double [ndofmax];
  
  g = new double [ndofcp];
  
  nullv (g,ndofcp);
  for (j=0;j<ns;j++){
    nullv (pp,ndofmax);
    coarse_local (j,pp,w);
    
    subv (pp,ff[j],ndofmas[j]);
    
    if (smst==dense_matrix)
      break;
    if (smst==skyline_matrix)
      smsky[j].ldl_feti_sky (d[j],pp,nrbmdom[j],se[j],zero);
    
    //  zacatek - streda 28.11.2007
    //  odecet skoku
    subv (d[j],jum[j],ndofmas[j]);
    //  konec - streda 28.11.2007
 
    local_coarse (j,d[j],g);
  }
  
  fprintf (out,"\n");
  for (i=0;i<ndofcp;i++){
    fprintf (out,"\n g  %20.15le",g[i]);
  }

  gamma = new double [hsize];
  u = new double [hsize];
  
  mtxv (h,g,u,ndofcp,hsize);
  fprintf (out,"\n");
  for (i=0;i<hsize;i++){
    fprintf (out,"\n u  %20.15le",u[i]);
  }

  mxv (h1,u,gamma,hsize,hsize);
  fprintf (out,"\n");
  for (i=0;i<hsize;i++){
    fprintf (out,"\n gamma  %20.15le",gamma[i]);
  }


  cmulv (-1.0,gamma,hsize);
  
  //for (j=0;j<ns;j++){
  //l=rbmadr[j];
  //for (i=0;i<nrbmdom[j];i++){
  //av[i] = gamma[l];  l++;
  //}
  //}
  
  //delete [] u;
  //delete [] gamma;
  //delete [] g;
  
  
  for (i=0;i<ns;i++){
    cmulv (-1.0,d[i],ndofmas[i]);
    
    nullv (av,6);
    l=rbmadr[i];
    for (j=0;j<nrbmdom[i];j++){
      av[j] = gamma[l];  l++;
    }

    fprintf (out,"\n");
    for (j=0;j<nrbmdom[i];j++){
      fprintf (out,"\n av %20.15le",av[j]);
    }

    for (j=0;j<ndofmas[i];j++){
      for (k=0;k<nrbmdom[i];k++){
	d[i][j]+=rbmdom[i][k*ndofmas[i]+j]*av[k];
      }
    }
  }
  

  //delete [] av;  delete [] pp;
}













/**
   function solves system of equations by the FETI method
   
   @param top - pointer to object with topology
   @param gm - %matrix of the system
   @param lhs - %vector of solution (left hand side)
   @param rhs - %vector of right hand side
   @param out - output stream
   
   JK, 16.9.2007
*/
void seqfeti::solve_system (gtopology *top,gmatrix *gm,
			    double *lhs,double *rhs,FILE *out)
{
  long i,j;
  double *h,*q,*hh,*ih,*ee,*lm;
  double **ff;
  time_t t1,t2,t3,t4,t5,t6;
  
  //  determination of the ndofmax
  det_ndofmax ();
  
  //  allocation of vectors of right hand sides on subdomains
  ff = new double* [ns];
  for (i=0;i<ns;i++){
    ff[i] = new double [ndofmas[i]];
    for (j=0;j<ndofmas[i];j++){
      ff[i][j]=0.0;
    }
  }
  
  //  allocation of vectors of right hand sides on subdomains
  for (i=0;i<ns;i++){
    globloc (rhs,ff[i],cndom[i],ndofmas[i]);
  }
  
  //  assembling of subdomain matrices
  subdomain_matrices (gm,out);
  
  t1 = time (NULL);
  
  //  computation of kernels / rigid body modes
  kernel (out);
  
  t2 = time (NULL);
  
  //  investigation of H matrix sizes
  hmatrixsize (out);
  
  fprintf (stdout,"\n\n hsize   %ld \n",hsize);

  if (hsize==0){
    fprintf (stdout,"\n\n hsize = 0 \n");
    fprintf (stderr,"\n\n hsize = 0 \n");
    abort ();
  }
  
  h = new double [ndofcp*hsize];
  q = new double [hsize];
  lm = new double [ndofcp];
  
  fprintf (out,"\n hsize   %ld",hsize);
  fprintf (out,"\n ndofcp   %ld",ndofcp);
  
  for (i=0;i<=ns;i++){
    fprintf (out,"\n rbmadr %4ld   %4ld",i,rbmadr[i]);
  }
  
  //  assembling of H matrix
  hmatrix (h);
  
  fprintf (out,"\n\n\n kontrola matice H \n");
  for (i=0;i<ndofcp;i++){
    fprintf (out,"\n %4ld",i);
    for (j=0;j<hsize;j++){
      fprintf (out,"  %f",h[i*hsize+j]);
    }
  }
  
  //  assembling of q vector
  qvector (q,ff);
  
  
  fprintf (out,"\n\n\n kontrola vektoru q \n");
  for (j=0;j<hsize;j++){
    fprintf (out,"\n %4ld   %f",j,q[j]);
  }
  
  t3 = time (NULL);
  
  //  H^T . H and (H^T . H)^{-1.0} evaluation
  ih = new double [hsize*hsize];
  hh = new double [hsize*hsize];
  ee = new double [hsize*hsize];
  for (i=0;i<hsize;i++){
    for (j=0;j<hsize;j++){
      ee[i*hsize+j]=0.0;
    }
    ee[i*hsize+i]=1.0;
  }
  
  mtxm (h,h,hh,ndofcp,hsize,hsize);
  
  fprintf (out,"\n\n\n kontrola matice HH \n");
  for (i=0;i<hsize;i++){
    fprintf (out,"\n %4ld ",i);
    for (j=0;j<hsize;j++){
      fprintf (out," %lf",hh[i*hsize+j]);
    }
  }
  
  gemp (hh,ih,ee,hsize,hsize,zero,1);
  
  //mtm (h,h,hh,ndofcp,hsize,hsize);
  //mm (hh,ih,ee,hsize,hsize,hsize);
  
  
  fprintf (out,"\n\n\n kontrola matice HHinv \n");
  for (i=0;i<hsize;i++){
    fprintf (out,"\n %4ld ",i);
    for (j=0;j<hsize;j++){
      fprintf (out," %lf",ih[i*hsize+j]);
    }
  }
  
  delete [] ee;
  
  //  modified conjugate gradient method
  
  t4 = time (NULL);
  
  
  mpcg (top,gm,lm,ff,q,h,ih,out);
  //mprcg (top,gm,lm,ff,q,h,ih,out);
  
  t5 = time (NULL);
  
  double **dd;
  dd = new double* [ns];
  for (i=0;i<ns;i++){
    dd[i] = new double [ndofmas[i]];
    for (j=0;j<ndofmas[i];j++){
      dd[i][j]=0.0;
    }
  }
  
  lagrmultdispl (top,gm,lm,dd,ff,h,ih,out);
  
  
  for (i=0;i<ns;i++){
    for (j=0;j<ndofmas[i];j++){
      lhs[cndom[i][j]-1]=dd[i][j];
    }
  }
  
  fprintf (out,"\n\n kontrola");
  for (i=0;i<ns;i++){
    fprintf (out,"\n");
    for (j=0;j<ndofmas[i];j++){
      fprintf (out,"\n dd %4ld %4ld     %lf",i,j,dd[i][j]);
    }
  }
  
  
  t6 = time (NULL);
  
  
  fprintf (out,"\n\n evaluation of kernel             %ld",t2-t1);
  fprintf (out,"\n\n matrix H computation             %ld",t3-t2);
  fprintf (out,"\n\n decomposition of H               %ld",t4-t3);
  fprintf (out,"\n\n modified conjug. gradient m.     %ld",t5-t4);
  fprintf (out,"\n\n displacements computation        %ld",t6-t5);

  fprintf (stdout,"\n\n evaluation of kernel             %ld",t2-t1);
  fprintf (stdout,"\n\n matrix H computation             %ld",t3-t2);
  fprintf (stdout,"\n\n decomposition of H               %ld",t4-t3);
  fprintf (stdout,"\n\n modified conjug. gradient m.     %ld",t5-t4);
  fprintf (stdout,"\n\n displacements computation        %ld",t6-t5);

  
  
  
  //  vectors of right hand sides on subdomains
  for (i=0;i<ns;i++){
    delete [] ff[i];
  }
  delete [] ff;

}
